from collections import Counter, OrderedDict
from sklearn.naive_bayes import GaussianNB, MultinomialNB, ComplementNB
from sklearn.ensemble import RandomForestClassifier, GradientBoostingClassifier
from sklearn.model_selection import RepeatedStratifiedKFold
from sklearn import svm
from sklearn.neural_network import MLPClassifier
from sklearn import metrics
from random import shuffle
import pickle
import os
import numpy as np
from .model_base import ModelBase
from ipclassifier.dataprep import RawFileProcessor



class ModelTrainer(ModelBase):

    PERIODS = (
        "15th-Century",
        "16th-Century",
        "17th-Century",
        "18th-Century",
        "19th-Century-(Romantic)",
        "19th-Century-(Victorian)"
    )
    SECTION_LENGTH = 100

    def __init__(self, feature_runner ,accented_words_file="master_word_list.pickle"):
        super().__init__()
        self.accented_words_file = os.path.join(os.path.dirname(__file__), accented_words_file)
        self.feature_runner = feature_runner

        self.main()

    
    def get_sections_per_period(self, period, iterations=1):
        print('get sections per period started...')

        filename = os.path.join(os.path.dirname(__file__), f"poems/_{period}.txt")
        rfp = RawFileProcessor(filename)
        contents = rfp.cleaned_contents

        for j in range(iterations):
            shuffle(contents)
            sectioned_contents = []
            sections = []
            for i,line in enumerate(contents):
                if i == 0: continue
                sections.append(line.rstrip("\n"))
                if i % self.SECTION_LENGTH == 0:
                    sectioned_contents.append(sections)
                    sections = []

            for i,section in enumerate(sectioned_contents):
                r = self.feature_runner(section)
                features = r.initial_process_contents(period)
                counter_dict = features["counter_dict"]
                rules_avg = features["rules_avg"]
                words_per_line = features["words_per_line"]
                avg_syllables_per_line = features["avg_syllables_per_line"]
                rule_0 = features["rule_0"]
                rule_1 = features["rule_1"]
                rule_2 = features["rule_2"]
                rule_3 = features["rule_3"]
                rule_4 = features["rule_4"]
                rule_5 = features["rule_5"]
                rule_6 = features["rule_6"]
                self.counter_dicts.append(counter_dict)
                self.other_features = [rules_avg, words_per_line, avg_syllables_per_line, rule_0, rule_1, rule_2, rule_3, rule_4, rule_5, rule_6] 


    def create_accented_word_feature(self, period):
        print('create accebted word feature started...')

        period_features = []
        for i,sect in enumerate(self.counter_dicts):
            sect = sorted([word for word in sect])
            sect_dict = OrderedDict(Counter(sect))
            sect_combined = OrderedDict()
            for k,v in self.ordered_accented_words.items():
                if k in sect_dict:
                    sect_combined[k] = 1
                else:
                    sect_combined[k] = 0
            one_hot_sect = [[float(v) for v in sect_combined.values()]]
            one_hot_sect[0].extend(self.other_features)
            one_hot_sect.append(period)
            period_features.append(one_hot_sect)

        return period_features


    def reset(self):
        print('reset started...')
        self.counter_dicts = []
        self.other_features = []


    def combine_all_period_features(self):
        print('combine all period features started...')
        flattened_all_period_features = [sect for period in self.all_period_features for sect in period]
        shuffle(flattened_all_period_features)
        return flattened_all_period_features


    def get_train_test_split(self, flattened_all_period_features):
        print('get train test split started...')
        X = [x[0] for x in flattened_all_period_features]
        y = [x[1] for x in flattened_all_period_features]

        size = len(X)
        print("size y", len(y), y)
        if size != len(y): raise Exception("X and y not same len")
        test_split_point = size // 10
        X_train = X[test_split_point:]
        y_train = y[test_split_point:]
        X_train_np = np.array(X_train)
        y_train_np = np.array(y_train)

        X_test = X[:test_split_point]
        y_test = y[:test_split_point]
        X_test_np = np.array(X_test)
        y_test_np = np.array(y_test)

        # print("saving train test pickle...")
        # with open("train_test_data.pickle", "wb") as f:
        #     pickle.dump({
        #     "X_test_np": X_test_np,
        #     "y_test_np": y_test_np,
        #     "X_train_np": X_train_np,
        #     "y_train_np": y_train_np
        # }, f)

        return {
            "X_test_np": X_test_np,
            "y_test_np": y_test_np,
            "X_train_np": X_train_np,
            "y_train_np": y_train_np
        }


    def train_model(self, train_test):
        print('train model started...')
        # models = [MultinomialNB, ComplementNB, MLPClassifier]
        # names = ["MultinomialNB", "ComplementNB", "MLPClassifier"]
        # for i,model in enumerate(models):
        #     print(str(model))
        #     print( len(train_test["X_train_np"]), len(train_test["y_train_np"]) )

        #     if names[i] == "ComplementNB":
        #         test_model = model(alpha=2.0)
        #     if names[i] == "MLPClassifier":
        #         test_model = model(activation="identity", alpha=1e-08, solver="lbfgs", hidden_layer_sizes=(100,), max_iter=2000)
        #     test_model = model()
        #     test_model.fit(train_test["X_train_np"], train_test["y_train_np"])
        test_model = ComplementNB(alpha=2.0)
        test_model.fit(train_test["X_train_np"], train_test["y_train_np"])
        with open(f'garbage.pickle', 'wb') as f:
            pickle.dump(test_model, f)


    def test_model(self, train_test):
        with open(f"garbage.pickle", 'rb') as f:
            test_model = pickle.load(f)
            predicted = test_model.predict(train_test["X_test_np"])
            print(metrics.classification_report(train_test["y_test_np"], predicted))
            print(metrics.confusion_matrix(train_test["y_test_np"], predicted))
            print(metrics.accuracy_score(train_test["y_test_np"], predicted))

            print("Accuracy on training set: {:.3f}".format(test_model.score(train_test["X_train_np"], train_test["y_train_np"])))
            print("Accuracy on test set: {:.3f}".format(test_model.score(train_test["X_test_np"], train_test["y_test_np"])))


    def main(self):
        print('main started...')
        self.create_accented_word_dict()
        for period in self.PERIODS:
            print("on period: ", period)
            self.get_sections_per_period(period)
            period_features = self.create_accented_word_feature(period)
            self.all_period_features.append(period_features)
            print("len all period features", len(self.all_period_features))
            self.reset()
        flattened_all_period_features = self.combine_all_period_features()
        print("len flattened all period features", len(flattened_all_period_features))
        train_test = self.get_train_test_split(flattened_all_period_features)
        self.train_model(train_test)
        self.test_model(train_test)
        




from os.path import exists
from .feature_runner import FeatureRunner
from ipclassifier.dataprep import RawFileProcessor
from ipclassifier.classifier import Classifier

##### use local copy of nltk_data if not elsewhere
import nltk
import os
local_path = os.path.join("".join(os.path.dirname(__file__).split('/runners')[0]),  "nltk_data")
nltk.data.path.append(local_path)



def classify_ip(filename):
    """
    Entrypoint for classifying a text file
    """

    if exists(filename):
        print("processing starting...")
        rfp = RawFileProcessor(filename)
        contents = rfp.cleaned_contents
        r = FeatureRunner(contents)
        features = r.initial_process_contents()
        c = Classifier(features)
        print(f"Your text is probably {c.guessed_period}")
        return c.guessed_period
    else:
        raise FileNotFoundError("Please make sure your filepath is correct.")




import re
from string import punctuation
from .token import Token



class Tokenizer():
    """
    Takes in a line of IP
    Returns list of Token instances
    """

    def __init__(self, lines, dicts):
        self._lines = lines
        self._remove = punctuation
        self._dicts = dicts


    def _tokenize_line(self, line):
        return re.split("[ \n!\"#$%&()*+,./:;<=>?@[\]^_`{|}~  \t]", line)


    def _make_initial_tokens(self):
        initial_tokens = [self._tokenize_line(line) for line in self._lines]
        return initial_tokens


    def _process_lines(self):
        initial_tokens = self._make_initial_tokens()
        cleaned_tokens = [ [self._clean(token) for token in line if token not in self._remove] for line in initial_tokens]
        return cleaned_tokens


    def _clean(self, word):
        word = word.lower()
        if word.endswith("'s"):
            word = word[:-2]
        return word


    def create_tokens(self):
        lines = self._process_lines()
        tokenized_lines = []
        for line in lines:
            tokenized_line = [Token(t, self._dicts) for t in line]
            if tokenized_line: tokenized_lines.append(tokenized_line)
        return tokenized_lines



    



from .spelling import SpellingNormalizer



class Compounds:
    """
    If a word token cannot be found in the cmudict, checks if it is composed of two words
    should_proceed_to_compound_analysis helps reduce false positives
    """

    def __init__(self, original_word, words, lemmatizer, uk_us_dict, regularize_dicts):
        self._original_word = original_word
        self._lemmatizer = lemmatizer
        self._words = words
        self._uk_us_dict = uk_us_dict
        self.regularize_dicts = regularize_dicts


    def _should_proceed_to_compound_analysis(self):
        if self._original_word in self._words or self._original_word[:-1] in self._words:
            return False
        return True


    def find_compound_in_wordlist(self, initial=2):
        dashed_word = self._handle_dashed_word()
        if dashed_word:
            return dashed_word

        if not self._should_proceed_to_compound_analysis():
            return None

        median = len(self._original_word) // 2
        current_split_idx = 0
        wordlist = self._words
        wnl = self._lemmatizer
        res = []

        for i in range(initial, len(self._original_word) - 2):
            left = SpellingNormalizer(self._original_word[:i], self._uk_us_dict, self.regularize_dicts).modernized_word[0] or self._original_word[:i]
            right = SpellingNormalizer(wnl.lemmatize(self._original_word[i:]), self._uk_us_dict, self.regularize_dicts).modernized_word[0] or wnl.lemmatize(self._original_word[i:])
            if left in wordlist and right in wordlist:
                if current_split_idx:
                    prior_distance = abs(current_split_idx - median)
                    current_distance = abs(i - median)
                    if current_distance <= prior_distance:
                        current_split_idx = i
                        res = [self._original_word[:current_split_idx], self._original_word[current_split_idx:]]    
                else:
                    current_split_idx = i
                    res = [self._original_word[:current_split_idx], self._original_word[current_split_idx:]]
        return res or None

    
    def _handle_dashed_word(self):
        if "-" in self._original_word:
            left, right = self._original_word.split("-")[0], "".join([*self._original_word.split('-')[1:]])
            left = SpellingNormalizer(left, self._uk_us_dict, self.regularize_dicts).modernized_word[0] or left
            right = SpellingNormalizer(self._lemmatizer.lemmatize(right), self._uk_us_dict, self.regularize_dicts).modernized_word[0] or self._lemmatizer.lemmatize(right)
            if left in self._words and right in self._words:
                return [left, right]




from .combinations_base import CombinationsBaseMixin
from .graph_base import GraphBase



class CombinationsGraph(CombinationsBaseMixin, GraphBase):
    """
    Uses CombinationsBaseMixin and GraphBase to construct new stress patterns for lines
    """

    def __init__(self, lines, product_possibles, is_odd_postition, target_stress):
        super().__init__(lines, product_possibles, is_odd_postition, target_stress)
        self.new_combinations = []
        self._all_paths = []
        self._path = []
        self._graph = {}

        self._main()


    def _final_processing(self):
        new_combinations = []
        for line in self._all_paths:
            new_line = [w[1] for w in line[1:]]
            new_combinations.append(tuple(new_line))
        self.new_combinations = new_combinations
        

    def _main(self):
        self._get_line_combinations()
        self._get_all_paths(0,len(self._graph) -1)
        self._final_processing()



import pickle
import os
import numpy as np
from .model_base import ModelBase



class Classifier(ModelBase):
    """
    Processes sample features to be consumed by the model
    Categorizes the sample
    """

    def __init__(self, features, accented_words_file="master_word_list.pickle"):
        super().__init__()
        self.features = features
        self.accented_words_file = os.path.join(os.path.dirname(__file__), accented_words_file)
        self.guessed_period = ''

        self.main()


    def combine_all_period_features(self):
        flattened_all_period_features = [sect for period in self.all_period_features for sect in period]
        return np.array(flattened_all_period_features)


    def guess_period(self, flattened_all_period_features):
        guess_set = []
        for m in ["MultinomialNB", "ComplementNB"]:
            filepath = os.path.join(os.path.dirname(__file__), f"models/{m}-test-model.pickle")
            with open(filepath, "rb") as f:
                model = pickle.load(f)
                guessed_period = model.predict(flattened_all_period_features)
                guess_set.append(guessed_period[0])
        self.guessed_period = guess_set[1] if guess_set[1] == "16th-Century" else guess_set[0]


    def main(self):
        self.create_accented_word_dict()
        self.get_sections_per_period()
        period_features = self.create_accented_word_feature()
        self.all_period_features.append(period_features)
        flattened_all_period_features = self.combine_all_period_features()
        self.guess_period(flattened_all_period_features)
        


import codecs, os, json, re
from nltk.corpus import words
from nltk.stem import WordNetLemmatizer
from nltk.corpus import cmudict as cmud
from .uk_american import UKAmerican



class DictsSingleton():
    """
    Contains main reference data for looking up tokens
    Sets some ad-hoc entries
    Singleton
    """
    
    _instance = None
    def __new__(cls):

        if cls._instance is None:
            cls._instance = super().__new__(cls)

            cls._instance.words = words.words()
            cls._instance.lemmatizer = WordNetLemmatizer()
            cls._instance.cmudict = cmud.dict()
            cls._instance.uk_us_dict = UKAmerican().uk_us_dict

            cls._instance.cmudict["us"] = [['AH1', 'S']]
            cls._instance.cmudict["heaven"] = [['HH', 'EH1', 'V', 'AH0', 'N'], ['HH', 'EH1', 'N']]
            cls._instance.cmudict["heavens"] = [['HH', 'EH1', 'V', 'AH0', 'N', 'Z'], ['HH', 'EH1', 'N', 'Z']]
            cls._instance.cmudict["heavenly"] = [['HH', 'EH1', 'V', 'AH0', 'N', 'L', 'IY0'], ['HH', 'EH1', 'N', 'L', 'IY0']]
            cls._instance.cmudict["choir"] = [['K', 'W', 'AY1', 'ER0'], ['K', 'W', 'AY1', 'R']]
            cls._instance.cmudict["choirs"] = [['K', 'W', 'AY1', 'ER0', 'Z'], ['K', 'W', 'AY1', 'R', 'Z']]
            cls._instance.cmudict["forsooth"] = [[ 'F', 'ER0', 'S', 'OW1', 'TH']]
            cls._instance.cmudict["moon"] = [['M', 'UW1', 'N']]
            cls._instance.cmudict["dryad"] = [['D', 'R', "AY1", "AE0", "D"]]
            cls._instance.cmudict["dryads"] = [['D', 'R', "AY1", "AE0", "D", "Z"]]
            cls._instance.cmudict["bene"] = [['B', 'EH1', 'N', 'AH0'], ['B', 'EH1', 'N']]
            cls._instance.cmudict["wherefore"] = [["W", "EH1", "R", "F", "ER0"]]
            cls._instance.cmudict["thereon"] = [["DH", "EH0", "R", "AH1", "N"]]
            cls._instance.cmudict["whereon"] = [["W", "EH0", "R", "AH1", "N"]]

            f = os.path.join(os.path.dirname(__file__), 'files/emspelling.json')
            with codecs.open(f, 'r', encoding='utf-8') as f:
                temp = f.read()
            g = os.path.join(os.path.dirname(__file__), 'files/decruft.json')
            with codecs.open(g, 'r', encoding='utf-8') as f:
                dec = f.read()
            decruft = json.loads(dec)
            cls._instance.regularize_dicts = {
                "decruftre_macron": {re.compile(k): v for k,v in decruft.items()},
                "decruftre": {re.compile(k): v for k,v in decruft.items() if "~" not in k},
                "dictionary": json.loads(temp)
            }
            
        return cls._instance




import re
from ipclassifier.regularize import Regularize



class SpellingNormalizer():
    """
    Accepts a word token which is not in the cmudict
    Checks:
        -Old spelling
        -Brittish spelling
        -Archaic 2P verb endings
        -Abbreviations with apostrophes
    """

    def __init__(self, unknown_word, uk_us_dict, regularize_dicts):
        self._unknown_word = unknown_word
        self._uk_us_dict = uk_us_dict
        self._regularize_dicts = regularize_dicts
        self.modernized_word = []
        self._apostrophe_position = ''
        self._has_apostrophe = False
        self._modernized = ''

        self._main()


    def _local_list(self):
        local_list = {
            "o’er": "or"
        }
        return local_list.get(self._unknown_word, None)


    def _get_modernized_spelling(self):
        self._modernized = Regularize(self._regularize_dicts).modernize(self._unknown_word) or self._brittish_converter() or self._local_list()
        self._apostrophe_check()
        if self._modernized is None:
            self._old_fashioned_check()
            self._handle_not_found()
        else:
            self.modernized_word = [self._modernized, self._has_apostrophe, self._apostrophe_position]

    
    def _brittish_converter(self):
        if self._unknown_word in self._uk_us_dict:
            return self._uk_us_dict[self._unknown_word]

    
    def _old_fashioned_check(self):
        if self._unknown_word.endswith("est") or self._unknown_word.endswith('eth') and len(self._unknown_word) > 5:
            self._modernized = self._unknown_word[:-3]


    def _handle_not_found(self):
        if len(self._unknown_word) == 0:
            self.modernized_word = [self._modernized, self._has_apostrophe, '']
        elif self._unknown_word[0] == "'":
            self.modernized_word = [self._handle_initial_apostrophe(), self._has_apostrophe, "initial"]
        elif self._unknown_word[-1] =="'":
            self.modernized_word = [self._handle_final_apostrophe(), self._has_apostrophe, "final"]
        elif "'" in self._unknown_word[1:-2]:
            self.modernized_word = [self._handle_medial_apostrophe(), self._has_apostrophe, "medial"]
        else:
            self.modernized_word = [self._modernized, self._has_apostrophe, '']


    def _apostrophe_check(self):
        if len(self._unknown_word) <= 1:
            return
        if self._unknown_word[-1] == "'":
            self._has_apostrophe = True
            self._apostrophe_position = "final" 
        elif "'" in self._unknown_word[0:-1]:
            self._has_apostrophe = True
            if self._unknown_word[0] == "'":
                self._apostrophe_position = "initial"
            else:
                self._apostrophe_position = "medial"


    def _handle_medial_apostrophe(self): # TODO: make more robust
        return re.sub("'", "e", self._unknown_word)


    def _handle_initial_apostrophe(self):
        return self._unknown_word[1:]
    

    def _handle_final_apostrophe(self):
        return self._unknown_word


    def _main(self):
        self._get_modernized_spelling()






import unittest
import os
from ipclassifier.dataprep import RawFileProcessor


class TestRawFileProcessor(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.correct = "Of Paradise so late their happy seat\n"
        prefix = os.path.join(os.path.dirname(__file__), "test_data")
        rfp = RawFileProcessor(os.path.join(prefix, "raw_files/raw_lines.txt"))
        cls.cleaned = rfp.cleaned_contents

    def test_remove_digits(self):
        self.assertEqual(self.cleaned[0], self.correct)
    
    def test_remove_double_dash(self):
        self.assertEqual(self.cleaned[1], self.correct)

    def test_remove_extra_spaces(self):
        self.assertEqual(self.cleaned[2], self.correct)

    def test_remove_tabs(self):
        self.assertEqual(self.cleaned[3], self.correct)

    def test_remove_punct_in_word(self):
        self.assertEqual(self.cleaned[4], self.correct)

    def test_remove_final_apostrophe(self):
        self.assertEqual(self.cleaned[5], self.correct)


if __name__ == "__main__":
    unittest.main()


from ipclassifier.runners import classify_ip, train_runner
from argparse import ArgumentParser


if __name__ == "__main__":
    parser = ArgumentParser(description="English Iambic Pentameter Period Classifier")
    parser.add_argument("-f", dest="filename", required=True,
                        help="Input .txt file.  Assumes the file is in user_input_poems/. \
                              File should contain newline-separated iambic pentameter. \
                              Performance is best with 100 or more lines. "
                       )
    parser.add_argument('--train', dest="train", action="store_true", help='Set to train a new model.')    
    args = parser.parse_args()
    if args.train:
        print('training...')
        train_runner()
    else:
        classify_ip(f"{args.filename}")



# based on https://github.com/jrladd/regularize
import re



class Regularize:
    """
    Lookes yfe a wourde bene founde inn liste
    """

    def __init__(self, regularize_dicts):
        self.decruftre_macron = regularize_dicts["decruftre_macron"]
        self.decruftre = regularize_dicts["decruftre"]
        self.dictionary = regularize_dicts["dictionary"]


    def decruftify(self, word):
        poss = []
        if "~" in word or r"\u0304" in word:
            for k,v in self.decruftre_macron.items():
                if re.search(k,word) and len(poss) == 0:
                    poss.append(re.sub(k,v,word))
                elif re.search(k,word) and len(poss) != 0:
                    poss.append(re.sub(k,v,poss[-1]))
        else:
            for k,v in self.decruftre.items():
                if re.search(k,word) and len(poss) == 0:
                    poss.append(re.sub(k,v,word))
                elif re.search(k,word) and len(poss) != 0:
                    poss.append(re.sub(k,v,poss[-1]))
        if len(poss) != 0:
            return poss[-1]
        else:
            return word

    def lookup(self, word):
        word = self.decruftify(word)
        if word in self.dictionary:
            return self.dictionary[word]
        wordlower = word.lower()
        if word == word.title() and wordlower in self.dictionary:
            return self.dictionary[wordlower].title()
        if word == word.upper() and wordlower in self.dictionary:
            return self.dictionary[wordlower].upper()
        return None

    def two_word_check(self, word):
        if (word.startswith("t'") or word.startswith("th'")) and word != "th'":
            words = word.split("'")
            firstword = self.lookup(words[0]+"'") if self.lookup(words[0]+"'") is not None else ''
            secondword = self.lookup(words[1]) if self.lookup(words[1]) is not None else ''
            combined = firstword+" "+secondword
            return combined.strip()
        else:
            return self.lookup(word)

    def modernize(self, word):
        return self.two_word_check(word)



from collections import Counter
from ipclassifier.token_processors import Tokenizer
from ipclassifier.iambic_line_processors import IambicLine
from ipclassifier.utils import DictsSingleton



class FeatureRunner():
    """
    Coordinates input processing and feature extraction
    """

    def __init__(self, sentences):
        self._sentences = sentences 
        self._dicts = DictsSingleton()
        

    @staticmethod
    def _get_stats(truth):
        rules = [int(x.split(', ')[1]) for x in truth]
        total = len(rules)
        res_dict: Counter = Counter(rules)
        return {k: v/total * 100 for k,v in res_dict.items()}
        

    def initial_process_contents(self, genre=None):
        truth = []
        truth_and_lines = []
        all_changed_words = []
        all_rules = []
        all_words_per_line = []
        all_syllables_per_line = []
        tokenizer = Tokenizer(self._sentences, self._dicts)
        line_tokens: str = tokenizer.create_tokens()
        for line in line_tokens:
            iambic_line = IambicLine(line)
            changed_words = iambic_line.line_facts["changed_words"]
            all_rules.append(iambic_line.line_facts["rules_applied"])
            all_words_per_line.append(iambic_line.line_facts["words_per_line"])
            all_syllables_per_line.append(iambic_line.line_facts["syllables_per_line"])
            if changed_words: all_changed_words += changed_words
            truth.append(str(iambic_line))
            truth_and_lines.append( (str(iambic_line), [str(tkn) for tkn in line] ))
        ratios = self._get_stats(truth)
        counter_dict = Counter(all_changed_words)
        return {
            "counter_dict":counter_dict,
            "rules_avg": sum(all_rules) / len(all_rules),
            "words_per_line": sum(all_words_per_line) / len(all_words_per_line),
            "avg_syllables_per_line": sum([s for y in all_syllables_per_line for s in y]) / len(all_syllables_per_line),
            "rule_0": ratios.get(0,0),
            "rule_1": ratios.get(1,0),
            "rule_2": ratios.get(2,0),
            "rule_3": ratios.get(3,0),
            "rule_4": ratios.get(4,0),
            "rule_5": ratios.get(5,0),
            "rule_6": ratios.get(6,0)
        }







import unittest
from ipclassifier.token_processors.phoneme_fsm import PhonemeFSM
from ipclassifier.utils import DictsSingleton


class TestPhonemeFSM(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.dicts = DictsSingleton()

    def test_old_spelling(self):
        fsm = PhonemeFSM('disceased', self.dicts)
        analyzed = fsm.final_phoneme_repr
        self.assertEqual(analyzed[0], ['D', 'IH0', 'Z', 'IY1', 'Z', 'D'])
        self.assertEqual(analyzed[1], ['D', 'IH0', 'Z', 'IY1', 'Z', 'EH0', 'D'])

    def test_basic_compound_intercepted(self):
        fsm = PhonemeFSM("lamplit", self.dicts)
        analyzed = fsm.final_phoneme_repr
        self.assertEqual(analyzed, [['AH1', 'AH0'], ['AH1', 'AH0']] )

    # def test_old_spelling_compound(self):
    #     fsm = PhonemeFSM("silver-smithes", self.dicts)
    #     analyzed = fsm.final_phoneme_repr
    #     self.assertEqual(analyzed, [['S', 'IH1', 'L', 'V', 'ER0', 'S', 'M', 'IH1', 'TH', 'S']])

    def test_dashed_compound(self):
        fsm = PhonemeFSM("sun-moon", self.dicts)
        analyzed = fsm.final_phoneme_repr
        self.assertEqual(analyzed, [['S', 'AH1', 'N', 'M', 'UW1', 'N']])  

    # def test_regular_compound(self):
    #     fsm = PhonemeFSM("silversmiths", self.dicts)
    #     analyzed = fsm.final_phoneme_repr
    #     self.assertEqual(analyzed, [['S', 'IH1', 'L', 'V', 'ER0', 'S', 'M', 'IH1', 'TH', 'S']])

    def test_regular_compound_ed(self):
        fsm = PhonemeFSM("moonfaced", self.dicts)
        analyzed = fsm.final_phoneme_repr
        self.assertEqual(analyzed, [
           ['AH1', 'AH0'], ['AH1', 'EH0', 'AH0'], ['AH1', 'AH0', 'AH0']
        ])

    def test_elided_the(self):
        fsm = PhonemeFSM("th'", self.dicts)
        analyzed = fsm.final_phoneme_repr
        self.assertEqual(analyzed, [['DH', 'AH0'], ['DH', 'AH1'], ['DH', 'IY0'], ['DH']])
        

if __name__ == "__main__":
    unittest.main()


import re
from unidecode import unidecode



class RawFileProcessor:

    def __init__(self, src_filename):
        self.cleaned_contents = []
        self._src_filename = src_filename
        self._raw_contents = ''

        self._main()


    def read_file(self):
        with open(f"{self._src_filename}", "r") as f:
            self._raw_contents = (line for line in f.readlines())


    def print_contents(self):
        for line in self._raw_contents:
            print(line)


    def write_file(self, dest_filename):
        with open(dest_filename, 'a') as f:
            f.writelines(self.cleaned_contents)


    def clean_contents(self):
        cleaned = []
        html = re.compile(r'<{1}[^>]*>{1}')
        punctuation = re.compile(r'([!"#$%&\()*+,.\/:;<=>?@[\]^_{|}~\d])|(\'+$)')
        dashes = re.compile(r'(\s{2,})+|(-{2,})+|(-\s+?$)')
        apostrophe_s = re.compile(r"[\'\’]s\s{1}")
        the = re.compile(r'th\'\w+')
        romans = re.compile(r'^[MXLICV]+$|CANTO.*[MXLICV]+|\[[mxlicv]+\]|^[^a-z]+$')
        ae = re.compile(r'[æÆ]')
        oe = re.compile(r'[œŒ]')

        for line in self._raw_contents:
            if len(line) < 2: continue
            cleaned_line = line.strip()
            cleaned_line = unidecode(cleaned_line)
            cleaned_line = re.sub(html, '', cleaned_line)
            cleaned_line = re.sub(punctuation, "", cleaned_line)
            cleaned_line = re.sub(dashes, " ", cleaned_line)
            cleaned_line = re.sub(apostrophe_s, " ", cleaned_line)
            cleaned_line = re.sub(the, "th' ", cleaned_line)
            cleaned_line = re.sub(romans, '', cleaned_line)
            cleaned_line = re.sub(ae, "ae", cleaned_line)
            cleaned_line = re.sub(oe, "oe", cleaned_line)
            cleaned_line = re.sub("`", "'", cleaned_line)
            cleaned.append(cleaned_line + "\n")
        self.cleaned_contents = cleaned


    def _main(self):
        self.read_file()
        self.clean_contents()
        